import java.util.Hashtable;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

public class ConnectionFactoryFromJNDI implements ConnectionFactory {

    private final String jndiName;
    private final Hashtable jndiEnv;

    public ConnectionFactoryFromJNDI(final String jndiName) {
        this(jndiName, null);
    }

    public ConnectionFactoryFromJNDI(final String jndiName, final Hashtable jndiEnv) {
        if (jndiName == null) {
            throw new NullPointerException("JNDI name cannot be null!");
        }
        this.jndiName = jndiName;
        this.jndiEnv = jndiEnv;
    }

    public Connection createConnection() throws JMSException {
        try {
            return lookup(jndiName, jndiEnv).createConnection();
        } catch (NamingException e) {
            throw new JMSException("JNDI lookup of ConnectionFactory failed: "+
                    e.toString(true));
        }
    }

    public Connection createConnection(final String username, final String password)
            throws JMSException {
        try {
            return lookup(jndiName, jndiEnv).createConnection(username, password);
        } catch (NamingException e) {
            throw new JMSException("JNDI lookup of ConnectionFactory failed: "+
                    e.toString(true));
        }
    }

    public static ConnectionFactory lookup(final String jndiName) throws NamingException {
        return lookup(jndiName, (Hashtable)null);
    }

    public static ConnectionFactory lookup(final String jndiName, final Hashtable jndiEnv)
            throws NamingException {
        final InitialContext ctx = jndiEnv == null ? new InitialContext() :
                                                     new InitialContext(jndiEnv);
        try {
            return lookup(jndiName, ctx);
        } finally {
            ctx.close();
        }
    }

    public static ConnectionFactory lookup(final String jndiName, final InitialContext ctx)
            throws NamingException {
        if (jndiName == null) {
            throw new NamingException("JMS ConnectionFactory name cannot be null!");
        }
        if (ctx == null) {
            throw new NamingException("JNDI InitialContext cannot be null!");
        }
        final Object object = ctx.lookup(jndiName);
        if (object == null) {
            throw new NamingException("JNDI lookup of JMS ConnectionFactory '"+
                    jndiName+"' returned null!");
        }
        if (object instanceof ConnectionFactory) {
            return (ConnectionFactory)object;
        }
        throw new NamingException("JNDI lookup of '"+jndiName+"' returned "+
                object.getClass()+" which is not javax.jms.ConnectionFactory!");
    }
}