class Pool<T> {

    private static class Entry<T> {
        public final T data;
        public final Entry<T> next;
        public Entry(final T data, final Entry<T> next) {
            this.data = data;
            this.next = next;
        }
    }

    private volatile Entry<T> head;

    public boolean isEmpty() {
        return head == null;
    }

    public void store(final T entry) {
        synchronized(this) {
            head = new Entry<T>(entry, head);
        }
    }

    public T get() {
        if (head == null) return null;
        synchronized(this) {
            final Entry<T> _head = head;
            if (_head == null) return null;
            head = _head.next;
            return _head.data;
        }
    }

}
