import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;

class SharedMessageConsumer implements MessageConsumer {

    private final MessageConsumer consumer;
    private boolean hasMessageListener;
    private volatile boolean closed;

    SharedMessageConsumer(final MessageConsumer consumer) {
        if (consumer == null) {
            throw new NullPointerException("MessageConsumer cannot be null!");
        }
        this.consumer = consumer;
    }

    private void checkClosed() throws javax.jms.IllegalStateException {
        if (closed) {
            throw new javax.jms.IllegalStateException("MessageConsumer has been closed");
        }
    }

    public synchronized void setMessageListener(final MessageListener mL) throws JMSException {
        checkClosed();
        consumer.setMessageListener(mL);
        hasMessageListener = mL != null;
    }

    public synchronized MessageListener getMessageListener() throws JMSException {
        checkClosed();
        return consumer.getMessageListener();
    }

    public String getMessageSelector() throws JMSException {
        checkClosed();
        return consumer.getMessageSelector();
    }

    public Message receive() throws JMSException {
        checkClosed();
        return consumer.receive();
    }

    public Message receive(final long timeout) throws JMSException {
        checkClosed();
        return consumer.receive(timeout);
    }

    public Message receiveNoWait() throws JMSException {
        checkClosed();
        return consumer.receiveNoWait();
    }

    public synchronized void close() throws JMSException {
        if (closed) return;
        closed = true;
        if (hasMessageListener) {
            consumer.setMessageListener(null);
            hasMessageListener = false;
        }
    }

}
